package cs4099;

import java.awt.Color;
import javax.swing.*;

/**
 * This class represents an individual playing card, which can have one of four suits and one of thirteen values.
 *
 * @author David Newton
 * @see Pile
 */

public class Card {
    
    //Attributes
    public int suit = 0; //Even suits black, odd suits red.
    public int value = 0;
    
    /**
     * Creates a new instance of Card.
     * @param suit The suit of the card (0-3: 0 = Spades, 1 = Hearts, 2 = Clubs, 3 = Diamonds)
     * @param value The value of the card (1-13)
     */
    public Card(int suit, int value) {
        this.suit = suit;
        this.value = value;
    }
    
    /**
     * Comparator method for cards.
     * @param card The card to compare to
     * @return True if the cards are equivalent, False if not.
     */
    public boolean equals(Card card) {
    	if (this.suit == card.suit && this.value == card.value) {
    		return(true);
    	}
    	else {
    		return(false);
    	}
    }
    
    /**
     * Converts the data held in this card to string form for output.
     */
    public String toString() {
        String strung = "";
        
        if (this.value > 1 && this.value < 11) {
             strung = Integer.toString(this.value);
        }
        else {
            switch (value) {
                case 1: strung = "A"; break;
                case 11: strung = "J"; break;
                case 12: strung = "Q"; break;
                case 13: strung = "K"; break;
            }
        }
        
        switch (suit) {
            case 0: strung += "S"; break;
            case 1: strung += "H"; break;
            case 2: strung += "C"; break;
            case 3: strung += "D"; break;
        }
        return strung;
    }
    
    /**
     * Builds a JLabel illustrating this card's properties.
     * @return A JLabel containing a suit image and textual value.
     */
    public JLabel getCardLabel() {
    	
    	JLabel label = new JLabel();
    	ImageIcon icon;
    	
    	//Suit/colour
    	if (this.suit == 0) {
			icon = new ImageIcon("./cards/suit_spade.png");
			label.setForeground(Color.BLACK);
		}
		else if (this.suit == 1) {
			icon = new ImageIcon("./cards/suit_heart.png");
			label.setForeground(Color.RED);
		}
		else if (this.suit == 2) {
			icon = new ImageIcon("./cards/suit_club.png");
			label.setForeground(Color.BLACK);
		}
		else {
			icon = new ImageIcon("./cards/suit_diamond.png");
			label.setForeground(Color.RED);
		}
    	label.setIcon(icon);
    	
    	//Value
    	switch (this.value) {
    		case 1: label.setText("A"); break;
    		case 11: label.setText("J"); break;
    		case 12: label.setText("Q"); break;
    		case 13: label.setText("K"); break;
    		default: label.setText(String.valueOf(this.value));
    	}
    	
    	return(label);
    }
    
    /**
     * A method that returns the image file that represents this card.
     * @return A JLabel containing the image.
     */
    public JLabel getCardImage() {
    	
    	JLabel label = new JLabel();
    	ImageIcon icon = new ImageIcon("./cards/" + suit + "-" + value + ".PNG");
    	label.setIcon(icon);
    	return(label);
    }
}
