package cs4099;

import javax.swing.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import java.util.*;

/**
 * This class deals with the running of a Solitaire game - the user manipulates the game from the GUI
 * drawn by this class.<br/><br/>
 * 
 * The class is an extension of a JFrame to provide a graphical user interface.
 * Most methods are private input handlers, and as such are not documented here.
 * 
 * @author David Newton
 */
public class Game extends JFrame implements ItemListener {
	
	private State rootstate;
	private State currentstate;
	
	private JLabel lblTitle1 = new JLabel("Solitaire Simulation, Specification and Solution System", JLabel.CENTER);
	private JLabel lblTitle2 = new JLabel("Written by David Newton", JLabel.CENTER);
	private JLabel lblTitle3 = new JLabel("CS4099 Senior Honours Project 2005-2006, University of St Andrews", JLabel.CENTER);
	private JLabel lblTitle4 = new JLabel("To begin, open an XML game file using the File menu.", JLabel.CENTER);
	
	private JLabel lblFrom = new JLabel("From");
	private JLabel lblTo = new JLabel("To");
	private JLabel lblNum = new JLabel("lblNum");
	private String gamename;
	
	private Button buttonSolve;
	private Checkbox checkSuggest;
	private Checkbox checkCompact;
	private Button buttonMove;
	private Button buttonUndo;
	private Button buttonDo;
	private Button buttonPlay;
	private Button buttonRewind;
	private JComboBox movebox;
	private Button buttonRetrieve;
	private Button buttonCancel;
	private Move[] moveboxcontents = {};
	
	private java.awt.List solutionlist;
	private LinkedList solutionpath = new LinkedList();
	private int solutionprogress = 0;
	
	private JTextField fieldFrom;
	private JTextField fieldTo;
	
	private Desktop desktop;
	
	private String gamefilename = "./!blank.xml";
	private Thread solvingthread;
	private Solver solver;
	
	//	The menu bar.
	private MenuBar menubar = new MenuBar();
	
	private Menu gameMenu = new Menu("Game");
	private Menu solverMenu = new Menu("Solver");
	private Menu debugMenu = new Menu("Debug");
	
	private MenuItem openItem = new MenuItem("Open...");
	private MenuItem restartItem = new MenuItem("Restart");
	private MenuItem redealItem = new MenuItem("Redeal");
	private MenuItem exitItem = new MenuItem("Exit");
	
	private MenuItem solveItem = new MenuItem("Solve...");
	private MenuItem cancelItem = new MenuItem("Cancel solver");
	private MenuItem retrieveItem = new MenuItem("Retrieve solution");
	
	private MenuItem hashItem = new MenuItem("State info");
	private MenuItem dumpItem = new MenuItem("Dump state");
	
	private StateWriter statewriter = new StateWriter();
	
	/**
	 * Constructor for creating a new Game window.
	 * @param rootstate The state with which to begin.
	 * @param xmlname The XML file to read by default (this can be changed by the user).
	 */
	public Game(State rootstate, String xmlname) {
		this.gamefilename = xmlname;
		this.gamename = rootstate.gamename;
		this.rootstate = rootstate.copy();
        this.currentstate = rootstate;
        currentstate.requiredcards = currentstate.findrequiredcards();
		initComponents();
	}
	
	/**
	 * Initializes the visual components of this class.
	 */
	public void initComponents() {
		
		//Essentials
		getContentPane().setLayout(new AbsoluteLayout());
        setDefaultCloseOperation(WindowConstants.EXIT_ON_CLOSE);
		
		//Initialise Solve button.
		buttonSolve = new Button();
		buttonSolve.setLabel("Solve");
		getContentPane().add(buttonSolve, new AbsoluteConstraints(750, 515, 40, 40));
		buttonSolve.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonSolveActionPerformed(evt);
	        }
	    });
		
//		Initialise Suggest checkbox.
		checkSuggest = new Checkbox();
		checkSuggest.setLabel("Suggest");
		getContentPane().add(checkSuggest, new AbsoluteConstraints(300, 540, 70, 20));
		checkSuggest.addItemListener(this);
		
//		Initialise Compact checkbox.
		checkCompact = new Checkbox();
		checkCompact.setLabel("Card View");
		getContentPane().add(checkCompact, new AbsoluteConstraints(400, 540, 100, 20));
		checkCompact.addItemListener(this);
		
//		Initialise Move button.
		buttonMove = new Button();
		buttonMove.setLabel("Move");
		getContentPane().add(buttonMove, new AbsoluteConstraints(300, 515, 40, 20));
		buttonMove.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonMoveActionPerformed(evt);
	        }
	    });

//		Initialise Rewind button.
		buttonRewind = new Button();
		buttonRewind.setLabel("Top");
		getContentPane().add(buttonRewind, new AbsoluteConstraints(810, 530, 45, 20));
		buttonRewind.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonRewindActionPerformed(evt);
	        }
	    });
		
//		Initialise Undo button.
		buttonUndo = new Button();
		buttonUndo.setLabel("Back");
		getContentPane().add(buttonUndo, new AbsoluteConstraints(855, 530, 45, 20));
		buttonUndo.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonUndoActionPerformed(evt);
	        }
	    });
		
		//Initialise Do button.
		buttonDo = new Button();
		buttonDo.setLabel("Fwd");
		getContentPane().add(buttonDo, new AbsoluteConstraints(900, 530, 45, 20));
		buttonDo.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonDoActionPerformed(evt);
	        }
	    });
		
//		Initialise Play button.
		buttonPlay = new Button();
		buttonPlay.setLabel("End");
		getContentPane().add(buttonPlay, new AbsoluteConstraints(945, 530, 45, 20));
		buttonPlay.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonPlayActionPerformed(evt);
	        }
	    });
		
//		Initialise Retrieve button.
		buttonRetrieve = new Button();
		buttonRetrieve.setLabel("Retrieve");
		getContentPane().add(buttonRetrieve, new AbsoluteConstraints(700, 515, 50, 20));
		buttonRetrieve.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonRetrieveActionPerformed(evt);
	        }
	    });
		
//		Initialise Cancel button.
		buttonCancel = new Button();
		buttonCancel.setLabel("Cancel");
		getContentPane().add(buttonCancel, new AbsoluteConstraints(700, 535, 50, 20));
		buttonCancel.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonCancelActionPerformed(evt);
	        }
	    });
		
//		Initialise Movebox.
		movebox = new JComboBox();
		getContentPane().add(movebox, new AbsoluteConstraints(30, 540, 250, 20));
		movebox.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            moveBoxActionPerformed(evt);
	        }
	    });
				
		//Initialise solution path list.
		solutionlist = new java.awt.List();
		getContentPane().add(solutionlist, new AbsoluteConstraints(810, 10, 180, 500));
		solutionlist.addItemListener(this);
		solutionlist.add("<<Start>>");
		solutionlist.add("<<End>>");
		solutionlist.select(solutionprogress);
		
		//Initialise Move fields.
		fieldFrom = new JTextField(3);
		getContentPane().add(fieldFrom, new AbsoluteConstraints(175, 515, 40, 20));
		fieldTo = new JTextField(3);
		getContentPane().add(fieldTo , new AbsoluteConstraints(240, 515, 40, 20));
		
		//Draw labels
		getContentPane().add(lblFrom, new AbsoluteConstraints(140, 515, 40, 20));
		getContentPane().add(lblTo, new AbsoluteConstraints(220, 515, 40, 20));
		
		
		//Special section for blank page
		getContentPane().add(lblTitle1, new AbsoluteConstraints(10, 200, 780, 20));
		getContentPane().add(lblTitle2, new AbsoluteConstraints(10, 230, 780, 20));
		getContentPane().add(lblTitle3, new AbsoluteConstraints(10, 250, 780, 20));
		getContentPane().add(lblTitle4, new AbsoluteConstraints(10, 290, 780, 20));
		if (currentstate.piles.size() > 0) {
			lblTitle1.setVisible(false);
			lblTitle2.setVisible(false);
			lblTitle3.setVisible(false);
			lblTitle4.setVisible(false);
		}
		
		//Draw the desktop
		desktop = new Desktop(currentstate, false);
		getContentPane().add(desktop, new AbsoluteConstraints(10, 10, 780, 500));
		
		//Draw the menu bar
		
		gameMenu.add(openItem);
		openItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            itemOpenActionPerformed(evt);
	        }
	    });
		
		gameMenu.addSeparator();
		
		gameMenu.add(restartItem);
		restartItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            itemRestartActionPerformed(evt);
	        }
	    });
		
		gameMenu.add(redealItem);
		redealItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            itemRedealActionPerformed(evt);
	        }
	    });
		
		gameMenu.addSeparator();
		
		gameMenu.add(exitItem);
		exitItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            System.exit(0);
	        }
	    });
		
		solverMenu.add(solveItem);
		solveItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonSolveActionPerformed(evt);
	        }
	    });
		
		solverMenu.add(cancelItem);
		cancelItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonCancelActionPerformed(evt);
	        }
	    });
		
		solverMenu.addSeparator();
		
		solverMenu.add(retrieveItem);
		retrieveItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            buttonRetrieveActionPerformed(evt);
	        }
	    });
		
		debugMenu.add(hashItem);
		hashItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            itemInfoActionPerformed(evt);
	        }
	    });
		
		debugMenu.add(dumpItem);
		dumpItem.addActionListener(new ActionListener() {
	        public void actionPerformed(ActionEvent evt) {
	            itemDumpActionPerformed(evt);
	        }
	    });
		
		menubar.add(gameMenu);
		menubar.add(solverMenu);
		menubar.add(debugMenu);
		
		this.setMenuBar(menubar);
		
		this.setTitle("Solitaire Specification, Simulation and Solution System - " + gamename);
		this.setSize(1000, 620);
		this.setVisible(true);
	}
	
	/**
	 * Updates the desktop by replacing it with a newly drawn version.
	 */
	private void updateDesktop() {
		solutionlist.select(solutionprogress);

		getContentPane().remove(desktop);
		desktop = new Desktop(currentstate, checkCompact.getState());
		getContentPane().add(desktop, new AbsoluteConstraints(10, 10, 780, 500));
		
		getContentPane().paintAll(getContentPane().getGraphics());
	}
	
	/**
	 * Opens a new XML file.
	 * @param evt
	 */
	private void itemOpenActionPerformed(ActionEvent evt) {
		JFileChooser chooser = new JFileChooser();
		chooser.setCurrentDirectory(new File("./xmls/"));
		FileFilter filter = new FileFilter();
		chooser.setFileFilter(filter);
        int returnVal = chooser.showOpenDialog(this);
        
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			gamefilename = chooser.getSelectedFile().getAbsolutePath();
			redealGame();
		}
	}
	
	/**
	 * Action for the Info item - display state information.
	 * @param evt Action event.
	 */
	private void itemInfoActionPerformed(ActionEvent evt) {
		//Report information on the current state.
		String msg = "";
		msg += ("State hashcode = " + currentstate.getSortedState().hashCode() + "\n");
		msg += ("State depth = " + (currentstate.depth) + "/" + (solutionlist.getItemCount()-2) + "\n");
		msg += ("Tactics : \n");
		for (int i = 0; i < currentstate.gametactics.length; i++) {
			msg += (currentstate.gametactics[i] + "\n");
		}
		JOptionPane.showMessageDialog(null, msg, "State information", JOptionPane.PLAIN_MESSAGE);
		updateDesktop();
	}
	
	/**
	 * Action for the Rewind button - return to the beginning of the solution path.
	 * @param evt Action event.
	 */
	private void buttonRewindActionPerformed(ActionEvent evt) {
		//Rewind the game to the beginning, keeping the solution path.
		while (currentstate.parent != null) {
			currentstate = currentstate.parent;
		}
		solutionprogress = 0;
		if (checkSuggest.getState() == true) {
			suggestMoves();
		}
		updateDesktop();
	}
	
	/**
	 * Action for the Solve button - run the solver from the current state.
	 * @param evt Action event.
	 */
	private void buttonSolveActionPerformed(ActionEvent evt) {
		
		//Prompt the user for the maximum depth to search.
		int maxdepth = 500;
		String enteredstring = "";
		
		try {
			enteredstring = JOptionPane.showInputDialog(null, "Enter maximum depth to search (default 500)", "Solve", JOptionPane.QUESTION_MESSAGE);
			
			//Return without starting if the entered string is null.
			if (enteredstring == null) {
				return;
			}
			
			maxdepth = Integer.parseInt(enteredstring);
		}
		catch (NumberFormatException e) {
			//Use the default if an invalid number was entered.
			maxdepth = 500;
		}

		//Now get rid of other running Solvers.
		if (solver != null) {
			solver.cancelled = true;
			solver.setVisible(false);
			solver = null;
			solvingthread = null;
		}
		
		solver = new Solver(currentstate, maxdepth);
		solvingthread = new Thread(solver);
		solvingthread.start();	
	}
	
	/**
	 * Action for the Retrieve button - get the solution path from the solver if a stopped
	 * one exists, or cancel it if it is running already.
	 * 
	 * @param evt Action event.
	 */
	private void buttonRetrieveActionPerformed(ActionEvent evt) {
		//Cancel the solver, then retrieve the solution path from it.
		if (solver == null) {
			JOptionPane.showMessageDialog(null, "No solver has been started!", "No solver", JOptionPane.WARNING_MESSAGE);
		}
		else {
			solver.cancelled = true;
			Move[] path = solver.solutionpath;
			if (path.length > 0) {
				solutionpath.clear();
				solutionlist.removeAll();
				solutionlist.add("<<Start>>");
				for (int i = 0; i < path.length; i++) {
					solutionpath.add(path[i]);
					solutionlist.add(path[i].toString());
				}
				solutionlist.add("<<End>>");
				solutionlist.select(solutionprogress);
			}
			else {
				JOptionPane.showMessageDialog(null, "Solver interrupted.\nClick Retrieve to display the best path found.", "Interrupted", JOptionPane.INFORMATION_MESSAGE);
			}
		}
	}
	
	/**
	 * Action for the Cancel button - cancel the solver if one exists.
	 * @param evt Action event.
	 */
	private void buttonCancelActionPerformed(ActionEvent evt) {
		//Cancel the solver, if one exists.
		if (solver == null || solver.cancelled) {
			JOptionPane.showMessageDialog(null, "The solver is not running!", "No solver", JOptionPane.WARNING_MESSAGE);
		}
		else {
			solver.cancelled = true;
			JOptionPane.showMessageDialog(null, "Solver interrupted.\nClick Retrieve to display the best path found.", "Interrupted", JOptionPane.INFORMATION_MESSAGE);
		}
	}
	
	/**
	 * Action for Redeal - redeal the game using the other method.
	 * @param evt Action event.
	 */
	private void itemRedealActionPerformed(ActionEvent evt) {
		//Restart the entire game, erasing all moves and dealing again.
		if (JOptionPane.showConfirmDialog(null, "Redeal game?", "Redeal", JOptionPane.YES_NO_OPTION) == JOptionPane.YES_OPTION) {
			redealGame();
		}
	}
	
	/**
	 * Action for the Restart button - go to the root state again, discarding the solution path.
	 * @param evt Action event.
	 */
	private void itemRestartActionPerformed(ActionEvent evt) {
		//Restart the game with the same layout.
		if (solver != null) {
			solver.setVisible(false);
		}
		solver = null;
		solvingthread = null;
		solutionlist.removeAll();
		solutionlist.add("<<Start>>");
    	solutionlist.add("<<End>>");
		//Restart by copying the root state from this game into the current state.
		currentstate = this.rootstate.copy();
		currentstate.requiredcards = currentstate.findrequiredcards();
		if (checkSuggest.getState() == true) {
			suggestMoves();
		}
		solutionprogress = 0;
		updateDesktop();
	}
	
	/**
	 * Method to perform actions based on the checkboxes in this class.
	 */
	public void itemStateChanged(ItemEvent e) {
		
		//Handles the "Suggest" checkbox.
		if (e.getSource() == checkSuggest) {
			if (checkSuggest.getState() == true) {
				//Immediately populate the list.
				suggestMoves();
			}
			else {
				movebox.removeAllItems();
			}
		}
		
		//Handles the Compact checkbox.
		if (e.getSource() == checkCompact) {
			updateDesktop();
		}
		
		//Handles the list.
		if (e.getSource() == solutionlist) {
			solutionlist.select(solutionprogress);
		}
	}
	
	/**
	 * Gathers tha available moves from the current state and places them in the suggested move box.
	 * @param evt Action event.
	 */
	private void suggestMoves() {
		
		//Fill the movebox with a set of suggested moves. Do this in reverse order, so that the best is first.
		movebox.removeAllItems();
		moveboxcontents = currentstate.findValidMoves();
		for (int i = moveboxcontents.length-1; i >= 0; i--) {
			movebox.addItem(moveboxcontents[i]);
		}
		
		//If there is now anything in the movebox, put the first option into the move fields.
		if (movebox.getItemCount() > 0) {
			fieldFrom.setText(String.valueOf(((Move)movebox.getItemAt(0)).sourcepileno));
			fieldTo.setText(String.valueOf(((Move)movebox.getItemAt(0)).targetpileno));
		}
		else {
			//There are no moves to display, but the game may have been solved.
			if (!gameSolved()) {
				JOptionPane.showMessageDialog(null, "There are no possible moves from this position.", "No moves", JOptionPane.INFORMATION_MESSAGE);
			}
		}
		
	}
	
	/**
	 * Action for the Move box - when an item is selected, place its values into the To/From fields.
	 * @param evt Action event.
	 */
	private void moveBoxActionPerformed(ActionEvent evt) {
		//When a move is selected, put its from and to values into the fields.
		try {
		fieldFrom.setText(String.valueOf(((Move)movebox.getSelectedItem()).sourcepileno));
		fieldTo.setText(String.valueOf(((Move)movebox.getSelectedItem()).targetpileno));
		}
		catch (Exception e) {
			//Do nothing.
		}
	}
	
	/**
	 * Action for the Undo button - step backward through the solution path.
	 * @param evt Action event.
	 */
	private void buttonUndoActionPerformed(ActionEvent evt) {
		
		//Undo the move by moving up to the last state if one is available.
		if (currentstate.parent == null) {
			JOptionPane.showMessageDialog(null, "Can't undo - you are at the start of game.", "Can't undo", JOptionPane.INFORMATION_MESSAGE);
    	}
    	else {
    		currentstate = currentstate.parent;
    		solutionprogress--;
    		if (checkSuggest.getState() == true) {
    			suggestMoves();
    		}
    	}
		updateDesktop();
	}
	
	/**
	 * Action for the Move button - attempt to perform the given move.
	 * @param evt Action event.
	 */
	private void buttonMoveActionPerformed(ActionEvent evt) {
		
		int movefrom = 0;
		int moveto = 0;
		try {
			movefrom = Integer.parseInt(fieldFrom.getText());
			moveto = Integer.parseInt(fieldTo.getText());
			Move movemade = attemptMove(movefrom, moveto);
			if (movemade != null) {
				
				//Remove anything above this in the solution list if the move is not equivalent to the next one to perform.
				if (solutionprogress < solutionlist.getItemCount()-2) {
					if (movemade.equals((Move)solutionpath.get(solutionprogress))) {
						solutionprogress++;
						updateDesktop();
					}
					else {
						//If the move is not equivalent, remove everything after this.
						while (!solutionlist.getItem(solutionlist.getSelectedIndex()+1).equals("<<End>>")) {
							solutionlist.remove(solutionlist.getSelectedIndex()+1);
							solutionpath.removeLast();
						}
						solutionpath.add(movemade);
						solutionlist.add(movemade.toString(), solutionprogress+1);
						solutionprogress++;
						updateDesktop();
					}
				}
				
				else {
					solutionpath.add(movemade);
					solutionlist.add(movemade.toString(), solutionprogress+1);
					solutionprogress++;
					updateDesktop();
				}
			}
		}
		catch (Exception e) {
			if (e.getClass() == NumberFormatException.class) {
				JOptionPane.showMessageDialog(null, "No move was entered.", "Info", JOptionPane.INFORMATION_MESSAGE);
			}
			if (e.getClass() == IndexOutOfBoundsException.class) {
				JOptionPane.showMessageDialog(null, "Invalid piles were entered.", "Info", JOptionPane.INFORMATION_MESSAGE);
			}
		}
			
	}
	
	/**
	 * Action for the Do button - step forward through the solution path.
	 * @param evt Action event.
	 */
	private void buttonDoActionPerformed(ActionEvent evt) {
		//Perform the next available move unless the end has been reached.
		if (!solutionlist.getItem(solutionlist.getSelectedIndex()+1).equals("<<End>>")) {
			int movefrom = ((Move)solutionpath.get(solutionprogress)).sourcepileno;
			int moveto = ((Move)solutionpath.get(solutionprogress)).targetpileno;
			attemptMove(movefrom, moveto);
			solutionprogress++;
			updateDesktop();
		}
		else {
			JOptionPane.showMessageDialog(null, "You have reached the end of your solution path", "No move", JOptionPane.INFORMATION_MESSAGE);
		}
	}
	
	/**
	 * Action for the Play button - play through a game until the end.
	 * @param evt Action event.
	 */
	private void buttonPlayActionPerformed(ActionEvent evt) {
		try {
			int speed = Integer.parseInt(JOptionPane.showInputDialog(null, "Enter the delay in ms", "Play through", JOptionPane.QUESTION_MESSAGE));
			if (speed > 0) {
				while (!solutionlist.getItem(solutionprogress+1).equals("<<End>>")) {
					int movefrom = ((Move)solutionpath.get(solutionprogress)).sourcepileno;
					int moveto = ((Move)solutionpath.get(solutionprogress)).targetpileno;
					attemptMove(movefrom, moveto);
					solutionprogress++;
					try {
						Thread.sleep(speed);
					}
					catch (Exception e) {
						
					}
				}
				solutionlist.select(solutionprogress);
			}
		}
		catch (NumberFormatException e) {
			
		}
	}
	
	/**
	 * Attempts to perform a move, adding a new state to the path if it is successful.
	 * @param movefrom The ID number of the pile to move from
	 * @param moveto The ID number of the pile to move to
	 * @return The move that was made, or null if the attempt was unsuccessful.
	 */
	private Move attemptMove(int movefrom, int moveto) {
		
		Move movemade = null;
		String canperform = currentstate.canPerformMove(((Pile)currentstate.piles.get(movefrom)), ((Pile)currentstate.piles.get(moveto)));
		
		if (canperform.equals("")) {
			
			//Copy the current state, then make changes to it.
			State oldstate = currentstate.copy();
			
			if (((Pile)currentstate.piles.get(movefrom)).isstock) {
				movemade = new Move(movefrom, 0, new Card(0, 0));
			}
			else {
				Card movedcard = ((Pile)currentstate.piles.get(movefrom)).getTopCard();
				movemade = new Move(movefrom, moveto, movedcard);
			}
			
			currentstate.sourcemove = movemade;
			currentstate.performMove(movefrom, moveto);
			currentstate.parent = oldstate;
			currentstate.depth = oldstate.depth + 1;
			currentstate.requiredcards = currentstate.findrequiredcards();
			
			//Repopulate the suggestion list if needed.
			if (checkSuggest.getState() == true) {
				suggestMoves();
			}
			
			//Update the display.
			updateDesktop();
			
			//Check for the new position being a solution.
			if (gameSolved()) {
				JOptionPane.showMessageDialog(null, "Game has been solved!", "Success", JOptionPane.INFORMATION_MESSAGE);
			}
			return (movemade);
		}
		else {
			JOptionPane.showMessageDialog(null, ("Moving from " + movefrom + " to " + moveto + " is invalid. " + canperform), "Info", JOptionPane.INFORMATION_MESSAGE);
			return (null);
		}
	}
	
	private boolean gameSolved() {
    	
    	Pile pile;
    	for (int i = 0; i < currentstate.piles.size(); i++) {
    		pile = (Pile)(currentstate.piles.get(i));
    		if (!pile.isfoundation && pile.stackpointer != 0) {
    			return(false);
    		}
    	}
    	
    	return (true);
    }
	
    /**
     * Redeals the entire game at random, resetting all progress on the current one.
     */
    private void redealGame() {
    	solutionlist.removeAll();
    	solutionlist.add("<<Start>>");
    	solutionlist.add("<<End>>");
		Interpreter interpreter = new Interpreter();
		currentstate = interpreter.setupGame(gamefilename);
		gamename = currentstate.gamename;
		rootstate = currentstate.copy();
		rootstate.parent = null;
		if (solver != null) {
			solver.setVisible(false);
		}
		solver = null;
		solvingthread = null;
		this.setTitle("Solitaire Specification, Simulation and Solution System - " + gamename);
		currentstate.requiredcards = currentstate.findrequiredcards();
		movebox.removeAllItems();
		
		fieldFrom.setText("");
		fieldTo.setText("");
		
		if (checkSuggest.getState() == true) {
			suggestMoves();
		}
		
		solutionprogress = 0;
		
		if (currentstate.piles.size() > 0) {
			lblTitle1.setVisible(false);
			lblTitle2.setVisible(false);
			lblTitle3.setVisible(false);
			lblTitle4.setVisible(false);
		}
		
		updateDesktop();
    }
    
    /**
     * Method to dump the current state into an XML element that describes the card layout as a series
     * of fixed cards.
     * @param evt
     */
    private void itemDumpActionPerformed(ActionEvent evt) {
    	statewriter.writeState(currentstate);
    }

}
