package cs4099;

/**
 * A Move is a specification of a move made, including the ID values of the source and target
 * piles along with a copy of the card that was moved.
 * 
 * @author David Newton
 * @see Pile
 * @see Card
 */
public class Move {
	
	public int sourcepileno;
	public int targetpileno;
	public Card movedcard;
	public int priority; //Used by the move finder to sort moves.
	
	/**
	 * Creates a new Move with the specified properties.
	 * @param sourcepile The ID of the source pile.
	 * @param targetpile The ID of the target pile.
	 * @param movedcard The card that was moved between the piles.
	 */
	public Move(int sourcepile, int targetpile, Card movedcard) {
		this.sourcepileno = sourcepile;
		this.targetpileno = targetpile;
		this.movedcard = movedcard;
	}
	
	/**
	 * Creates a String representation of this move, for use in the Suggest list.
	 */
	public String toString() {
		if (this.movedcard.value == 0) {
			//This is an unstocking move.
			return ("Unstock from pile " + this.sourcepileno);
		}
		else {
			return (this.movedcard.toString() + " from pile " + sourcepileno + " to pile " + targetpileno);
		}
	}
	
	/**
	 * Compares this Move to another, returning true if the moves are equal.
	 * @param othermove The move to compare to.
	 * @return true if the moves have the same source and target piles, and move the same card.
	 */
	public boolean equals(Move othermove) {
		if (!this.movedcard.equals(othermove.movedcard)) {return false;}
		if (this.sourcepileno != othermove.sourcepileno) {return false;}
		if (this.targetpileno != othermove.targetpileno) {return false;}
		else {return true;}
	}
}
