package cs4099;

import java.io.File;
import java.io.FileWriter;

import javax.swing.JOptionPane;

/**
 * The StateWriter outputs a representation of a state to an XML file so that it can be read
 * at a later time.
 * 
 * @author David Newton
 * @see State
 */
public class StateWriter {

	/**
	 * Constructor (empty).
	 *
	 */
	public StateWriter() {
	}
	
	/**
	 * Writes a state to the file "./dump.xml" in the expected XML format, so that it can be restored by using the Open command.
	 * @param state The state to write to the file.
	 */
	public void writeState(State state) {
    	File outputfile = new File("./!dump.xml");
    	String strung = "";
    	
    	//Header and game name/type
    	strung += "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    	strung += "<game name=\"Dumped State\" decks=\"" + state.countCards()/52 + "\">\n\n";
    	
    	//Iterate through all piles and add their properties
    	//Exclude the properties related to dealing, as they are not needed
    	
    	strung += "<piles>\n";
    	for (int i = 0; i < state.piles.size(); i++) {
    		
    		Pile pile = (Pile)state.piles.get(i);
    		
    		strung += "<pile id=\"" + pile.id + "\" ";
    		strung += "capacity=\"" + pile.contents.length + "\" ";
    		strung += "order=\"" + pile.order + "\" ";
    		strung += "uniquesuit=\"" + pile.uniquesuit + "\" ";
    		strung += "stockcapacity=\"" + pile.stockcapacity + "\" ";
    		strung += "startvalue=\"" + pile.startvalue + "\" ";
    		strung += "stocknumber=\"" + pile.stocknumber + "\" ";
    		
    		strung += "alternatecolours=\"" + pile.alternatecolours + "\" ";
    		strung += "samesuit=\"" + pile.samesuit + "\" ";
    		strung += "removable=\"" + pile.removable + "\" ";
    		strung += "addable=\"" + pile.addable + "\" ";
    		strung += "isfoundation=\"" + pile.isfoundation + "\" ";
    		strung += "isstock=\"" + pile.isstock + "\" ";
    		strung += "oneaway=\"" + pile.oneaway + "\" ";
    		strung += "oneawaynowrap=\"" + pile.oneawaynowrap + "\" ";
    		strung += "displayx=\"" + pile.displayx + "\" ";
    		strung += "displayy=\"" + pile.displayy + "\" ";
    		strung += "straightstack=\"" + pile.straightstack + "\" ";
    		strung += "backwards=\"" + pile.backwards + "\" ";
    		strung += "fourteens=\"" + pile.fourteens + "\" ";
    		
    		strung += "/>\n";
    	}
    	    	
    	strung += "</piles>\n\n";
    	strung += "<fixedcards>\n";
    	
    	//Place information on all cards in the game into this section.
    	for (int i = 0; i < state.piles.size(); i++) {
    		//Get the pile.
    		Pile pile = (Pile)state.piles.get(i);
    		
    		//Iterate through it and add the card information to the string.
    		for (int j = 0; j < pile.stackpointer; j++) {
    			strung += ("<card pile=\"" + i + "\" suit=\"" + pile.contents[j].suit + "\" value=\"" + pile.contents[j].value + "\" />\n");    			
    		}
    	}
    	strung += "</fixedcards>\n\n";
    	
    	//Now export the tactics from the current state.
    	strung += "<tactics>\n";
    	
    	for (int i = 0; i < state.gametactics.length; i++) {
    		strung += "<tactic name=\"" + state.gametactics[i] + "\"/>\n";
    	}
    	
    	strung += "</tactics>\n";
    	strung += "</game>\n";
    	
    	//Write the file.
    	try {
    		FileWriter filewriter = new FileWriter(outputfile);
    		filewriter.write(strung);
    		filewriter.close();
    		JOptionPane.showMessageDialog(null, "State written successfully to ./!dump.xml", "Dumped", JOptionPane.INFORMATION_MESSAGE);
    	}
    	catch (Exception e) {
    		JOptionPane.showMessageDialog(null, "Error writing to file.", "I/O", JOptionPane.ERROR_MESSAGE);
    	}
    }
}
